#include "Tfillable.h"
#include "Ttexture.h"

ImplementSimpleObjAttr(Tfillable, WORD,  FillMode)
ImplementBoolObjAttr(Tfillable, Texture)
ImplementSimpleObjAttr(Tfillable, int,   TextureIndex)
ImplementBoolObjAttr(Tfillable, AlphaBlending)	


Tfillable::~Tfillable() {}

Tfillable::Tfillable() : 
	fFillMode(D3DFILL_SOLID),
	fTexture(false),
	fTextureIndex(-1),
	fAlphaBlending(false)
	{}


void
Tfillable::SetObjectD3DAttributes(Td3dDrawable &d, LPDIRECT3DDEVICE9 device)
{
	//
	// Calls the parent class to do its work
	Tshadable::SetObjectD3DAttributes(d, device);

	// Set fill mode
	device->SetRenderState(D3DRS_FILLMODE, fFillMode);

	// Now, file texture ...
	if (TextureIsEnabled()) {
		Ttexture *t = d.GetTexture(fTextureIndex);
		if (t != NULL) {
			t->ActivateTexture();	
			// now to support blending ...
			if (AlphaBlendingIsEnabled()) {
						// 1. enable the blending
				device->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);

					// 2. define what we want to do ...
					//
					//   OurColor * (AlphaFromTexture) + (FrameBufferColor * (1-AlphaFromTexture) )
					//
				device->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
				device->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
			}
		}
	}
}


void
Tfillable::UnSetObjectD3DAttributes(Td3dDrawable &d, LPDIRECT3DDEVICE9 device)
{
	//
	// Calls the parent class to do its work
	Tshadable::UnSetObjectD3DAttributes(d, device);

	// Now, switch off file texture ... if we had switched it on ...
	if (TextureIsEnabled()) {
		Ttexture *t = d.GetTexture(fTextureIndex);
		if (t != NULL) {
			t->DeActivateTexture();	
			// now to support blending ...
			if (AlphaBlendingIsEnabled()) {
				// reset blending state
				device->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
			}
		}
	}
}
