#include "TxformInfo.h"

static D3DXVECTOR3 kZAxis(0.0f, 0.0f, 1.0f);

TxformInfo::TxformInfo() {
	ResetTransformation();
}

void TxformInfo::ResetTransformation() {
	fMove.x = fMove.y = fMove.z = 0;
	fScale.x = fScale.y = fScale.z = 1;
	fRotation = 0.0f;
}

void TxformInfo::SetUpD3DXformStack(ID3DXMatrixStack *stack) const {
	stack->TranslateLocal(fMove.x, fMove.y, 0.0f);
	stack->RotateAxisLocal(&kZAxis, fRotation);
	stack->ScaleLocal(fScale.x, fScale.y, 1.0f);
}

float TxformInfo::MoveX() const { return fMove.x; }
float TxformInfo::MoveY() const { return fMove.y; }
float TxformInfo::MoveZ() const { return fMove.z; }
const D3DXVECTOR3& TxformInfo::Movement() const { return fMove; }

float TxformInfo::ScaleX() const { return fScale.x; }
float TxformInfo::ScaleY() const { return fScale.y; }
float TxformInfo::ScaleZ() const { return fScale.z; }
const D3DXVECTOR3& TxformInfo::Scale() const { return fScale; }

float TxformInfo::RotationInRadian() const { return fRotation; }
float TxformInfo::RotationInDegree() const { return fRotation * 180.0f / M_PI; }


void TxformInfo::SetMoveTo(const D3DXVECTOR3 &d) {
		fMove = d; }
void TxformInfo::SetMoveTo(float x, float y, float z) {
		fMove.x = x;
		fMove.y = y;
		fMove.z = z; }
void TxformInfo::UpdateMovementBy(const D3DXVECTOR3 &v) { fMove += v; }
void TxformInfo::UpdateMovementBy(float x, float y, float z) {
		fMove.x += x;
		fMove.y += y; 
		fMove.z += z; }

void TxformInfo::SetScaleTo(const D3DXVECTOR3 &s) { 
		fScale = s; }
void TxformInfo::SetScaleTo(float s) { 
		fScale.x = s;
		fScale.y = s; 
		fScale.z = s; }
void TxformInfo::SetScaleXTo(float x) { fScale.x = x; }
void TxformInfo::SetScaleYTo(float y) { fScale.y = y; }
void TxformInfo::SetScaleZTo(float z) { fScale.z = z; }
void TxformInfo::SetScaleTo(float x, float y, float z) {
		fScale.x = x;
		fScale.y = y;
		fScale.z = z; }
void TxformInfo::UpdateScaleBy(float s) {
		fScale.x *= s;
		fScale.y *= s;
		fScale.z *= s; }
void TxformInfo::UpdateScaleBy(float x, float y, float z) {
		fScale.x *= x;
		fScale.y *= y;
		fScale.z *= z; }

void TxformInfo::SetRotationToRadian(float theta) {	fRotation = theta; }
void TxformInfo::SetRotationToDegree(float degree) { fRotation = degree * M_PI / 180.0f; }
void TxformInfo::UpdateRotationByRadian(float theta) { fRotation += theta; }
void TxformInfo::UpdateRotationByDegree(float degree) { fRotation += (degree * M_PI / 180.0f); }